@@ -1,5 +1,3 @@
-#  pbltest pgtest
-
 #### pbgtest
 
 pbgtest <- function (x, ...) 
@@ -7,35 +5,39 @@ pbgtest <- function (x, ...)
     UseMethod("pbgtest")
 }
 
-pbgtest.formula<-function(x, data, model="random",order=NULL, index=NULL, ...) {
+pbgtest.formula<-function(x, ..., order = NULL) {
   ## formula method for pbgtest;
-  ## defaults to a RE model
-  mymod <- plm(formula=x, data=data, model=model, index=index,...)
-  pbgtest(mymod,order)
+  ## defaults to a pooling model
+  cl <- match.call(expand.dots = TRUE)
+  if (names(cl)[3] == "") names(cl)[3] <- "data"
+  if (is.null(cl$model)) cl$model <- "pooling"
+  names(cl)[2] <- "formula"
+  m <- match(plm.arg,names(cl),0)
+  cl <- cl[c(1,m)]
+  cl[[1]] <- as.name("plm")
+  plm.model <- eval(cl,sys.frame(which=length(sys.calls())))
+  pbgtest(plm.model, order = order, ...)
 }
 
 pbgtest.panelmodel<-function(x, order = NULL, ...) {
   ## residual serial correlation test based on the residuals of the demeaned
   ## model (see Wooldridge p.288) and the regular bgtest() in {lmtest}
 
-
   ## structure:
   ## 1: take demeaned data from 'plm' object
   ## 2: est. auxiliary model by OLS on demeaned data
   ## 3: apply bgtest() to auxiliary model and return the result
 
   ## retrieve demeaned data
-  demX <- x$model[[2]]
-  demy <- x$model[[1]]
+  demX <- model.matrix(x)
+  demy <- model.response(model.frame(x))
   
   ## ...and group numerosities
-  Ti <- attr(x, "pdim")$Tint$Ti
-  
+  Ti <- pdim(x)$Tint$Ti
   ## set lag order to minimum group numerosity if not specified by user
   ## (check whether this is sensible)
   
   if(is.null(order)) order <- min(Ti)
-  
   ## bg test on the demeaned model:
   
   ## check package availability and load if necessary
@@ -43,18 +45,35 @@ pbgtest.panelmodel<-function(x, order = NULL, ...) {
   if(!lm.ok) stop("package lmtest is needed but not available")
   
   ## bgtest is the bgtest, exception made for the method attribute
-  bgtest <- bgtest(lm(demy~demX-1),order=order)
+  dots <- match.call()[["..."]]
+  if (!is.null(dots$type)) type <- dots$type else type <- "Chisq"
+  if (!is.null(dots$order.by)) order.by <- dots$order.by else order.by <- NULL
+  lm.mod <- lm(demy~demX-1)
+  bgtest <- bgtest(lm.mod, order = order, type = type, order.by = order.by)
   bgtest$method <- "Breusch-Godfrey/Wooldridge test for serial correlation in panel models"
   bgtest$alternative <- "serial correlation in idiosyncratic errors"
   bgtest$data.name <- paste(deparse(x$call$formula))
-  names(bgtest$statistic) <- "chisq"
-  # is it really working ? I'm wondering how the order of the test is taken into account ???
+  names(bgtest$statistic) <- if(length(bgtest$parameter)==1) "chisq" else "F"
   return(bgtest)
 }
 
 ### pwtest
 
-pwtest <- function(x, data, effect = c("individual","time"), index=NULL, ...) {
+pwtest <- function(x, ...){
+  UseMethod("pwtest")
+}
+
+pwtest.formula <- function(x, data, ...) {
+  cl <- match.call(expand.dots = TRUE)
+  if (names(cl)[3] == "") names(cl)[3] <- "data"
+  if (is.null(cl$model)) cl$model <- "pooling"
+  if (cl$model != "pooling") stop("pwtest only relevant for pooling models")
+  names(cl)[2] <- "formula"
+  m <- match(plm.arg,names(cl),0)
+  cl <- cl[c(1,m)]
+  cl[[1]] <- as.name("plm")
+  plm.model <- eval(cl,sys.frame(which=length(sys.calls())))
+  pwtest(plm.model)
 
   ## "RE" test à la Wooldridge, see 10.4.4
   ## (basically the scaled and standardized estimator for sigma from REmod)
@@ -65,144 +84,148 @@ pwtest <- function(x, data, effect = c("individual","time"), index=NULL, ...) {
 
   ######### from here generic testing interface from
   ######### plm to my code
+}
 
+pwtest.panelmodel <- function(x, ...){
   ## tind is actually not needed here
-
-  data <- plm.data(data,index=index)
-
+  if (describe(x, "model") != "pooling") stop("pwtest only relevant for pooling models")
+  effect <- describe(x, "effect")
+  data <- model.frame(x)
   ## extract indices
-  indices <- list(uno=as.numeric(data[,1]),
-                due=as.numeric(data[,2])
-                )  
 
   ## if effect="individual" std., else swap
-  myinds <- switch(match.arg(effect),
-          individual=indices,
-          time=list(indices[[2]],indices[[1]])
-          )
-
-  index <- myinds[[1]]
-  tindex <- myinds[[2]]
-
-  ## reduce X,y to model matrix values (no NAs)
-  X <- model.matrix(x,data=data)
-  y <- model.response(model.frame(x,data=data))
-  ## reduce index accordingly
-  names(index) <- row.names(data)
-  ind <- index[which(names(index)%in%row.names(X))]
-
-  ######### till here. 
-  ######### Add ordering here if needed.
-
-  ## this doesn't need ordering of obs.
 
+  if (effect == "individual"){
+    index <- data[["(id)"]]
+    tindex <- data[["(time)"]]
+  }
+  else{
+    index <- data[["(time)"]]
+    tindex <- data[["(id)"]]
+  }
   ## det. number of groups and df
-  n <- length(unique(ind))
-  k <- dim(X)[[2]]
-  ## det. max. group numerosity
-  t <- max(tapply(X[,1],ind,length))
+  n <- length(unique(index))
+  X <- model.matrix(x)
+
+  k <- ncol(X)
   ## det. total number of obs. (robust vs. unbalanced panels)
-  nT <- length(ind)
+  nT <- nrow(X)
+  ## det. max. group numerosity
+  t <- max(tapply(X[,1],index,length))
 
   ## ref. Wooldridge, p.264
     
-    ## estimate pooled model and extract resids
-    poolmod <- lm.fit(X,y)
-    u <- resid(poolmod)
+  ## extract resids
+  u <- resid(x)
 
-    ## est. random effect variance
-    ## "pre-allocate" an empty list of length n
-    tres <- vector("list", n)
+  ## est. random effect variance
+  ## "pre-allocate" an empty list of length n
+  tres <- vector("list", n)
 
-    ## list of n "empirical omega-blocks"
-    ## with averages of xproducts of t(i) residuals
-    ## for each group 1..n 
-    ## (possibly different sizes if unbal., thus a list
-    ## and thus, unlike Wooldridge (eq.10.37), ve divide 
-    ## every block by *his* t(t-1)/2)
-    unind <- unique(ind)
-    for(i in 1:n) {
-      ut <- u[ind==unind[i]]
-      tres[[i]] <- ut%o%ut
-      }
+  ## list of n "empirical omega-blocks"
+  ## with averages of xproducts of t(i) residuals
+  ## for each group 1..n 
+  ## (possibly different sizes if unbal., thus a list
+  ## and thus, unlike Wooldridge (eq.10.37), ve divide 
+  ## every block by *his* t(t-1)/2)
+#  unind <- unique(ind)
+  unind <- unique(index) # ????
+ 
+  for(i in 1:n) {
+    ut <- u[index == unind[i]]
+    tres[[i]] <- ut%o%ut
+  }
 
-    ## sum over all upper triangles of emp. omega blocks:
-    ## define aux. function
-    uptrisum <- function(x) {
-              uts <- sum(x[upper.tri(x,diag=FALSE)])
-              return(uts)}
+  ## sum over all upper triangles of emp. omega blocks:
+  ## define aux. function
+  uptrisum <- function(x) {
+    uts <- sum(x[upper.tri(x,diag=FALSE)])
+    return(uts)}
+  
+  ## det. # of upper triangle members (n*t(t-1)/2 if balanced)
+  ti <- sapply(tres, function(x) dim(x)[[1]])
+  uptrinum <- sum(ti*(ti-1)/2)  # don't need this!!
 
-    ## det. # of upper triangle members (n*t(t-1)/2 if balanced)
-    ti <- sapply(tres, function(x) dim(x)[[1]])
-    uptrinum <- sum(ti*(ti-1)/2)  # don't need this!!
-
-    ## ...apply to list and sum over resulting vector (df corrected)
-    W <- sum(sapply(tres,uptrisum)) # /sqrt(n) simplifies out
-
-    ## calculate se(Wstat) as in 10.40
-    seW <- sqrt( sum( sapply(tres,uptrisum)^2 ) )
-
-    ## NB should we apply a df correction here, maybe that of the standard
-    ## RE estimator? (see page 261) 
-
-    Wstat <- W/seW
-    names(Wstat) <- "z"
-    pW <- 2*pnorm(abs(Wstat),lower.tail=FALSE) # unlike LM, test is two-tailed!
+  ## ...apply to list and sum over resulting vector (df corrected)
+  W <- sum(sapply(tres,uptrisum)) # /sqrt(n) simplifies out
+  
+  ## calculate se(Wstat) as in 10.40
+  seW <- sqrt( sum( sapply(tres,uptrisum)^2 ) )
+  
+  ## NB should we apply a df correction here, maybe that of the standard
+  ## RE estimator? (see page 261) 
 
+  Wstat <- W/seW
+  names(Wstat) <- "z"
+  pW <- 2*pnorm(abs(Wstat),lower.tail=FALSE) # unlike LM, test is two-tailed!
+  
   ##(insert usual htest features)
   dname <- paste(deparse(substitute(formula)))
   RVAL <- list(statistic = Wstat, parameter = NULL,
                method = paste("Wooldridge's test for unobserved ",
-                               match.arg(effect),"effects "),
+                 effect,"effects "),
                alternative = "unobserved effect",
                p.value = pW,
                data.name =   dname)
   class(RVAL) <- "htest"
   return(RVAL)
-
+  
   }
 
 
 ### pwartest
 
-pwartest <- function(x, data, index=NULL, ...) {
+pwartest <- function(x, ...){
+  UseMethod("pwartest")
+}
+
+pwartest.formula <- function(x,  data, ...) {
   ## small-sample serial correlation test for FE models
   ## ref.: Wooldridge (2003) 10.5.4 
   if(!require(car)) stop("Library 'car' is needed")
-  data <- plm.data(data,index=index)
-  ## fetch within residuals
-  femod <- plm(x,data,model="within")
-  FEres <- resid(femod)
 
+  cl <- match.call(expand.dots = TRUE)
+  if (is.null(cl$model)) cl$model <- "within"
+  if (cl$model != "within") stop("pwartest only relevant for within models")
+  if (names(cl)[3] == "") names(cl)[3] <- "data"
+  names(cl)[2] <- "formula"
+  m <- match(plm.arg,names(cl),0)
+  cl <- cl[c(1,m)]
+  cl[[1]] <- as.name("plm")
+  plm.model <- eval(cl,sys.frame(which=length(sys.calls())))
+  pwartest(plm.model)
+}
+
+pwartest.panelmodel <- function(x, ...){
+  FEres <- resid(x)
+  data <- model.frame(x)
+
+  if (describe(x, "model") != "within") stop("pwartest only relevant for within models")
+  
   ## this is a bug fix for incorrect naming of the "data" attr.
   ## for the pseries in pdata.frame()
-  attr(FEres, "data") <- paste(deparse(substitute(data)))
-#  FEres.1 <- lag(FEres,k=1)
+  
+  attr(FEres,"data") <- NULL
   N <- length(FEres)
-#  FEres.1 <- c(NA,FEres[2:N]-FEres[1:(N-1)])
   FEres.1 <- c(NA,FEres[1:(N-1)])
-  id <- as.numeric(femod$indexes$id)
-  lagid <- id-c(NA,id[1:(N-1)])
+  id <- data[["(id)"]]
+  lagid <- as.numeric(id)-c(NA,as.numeric(id)[1:(N-1)])
   FEres.1[lagid!=0] <- NA
-  data$FEres <- FEres
-  data$FEres.1 <- FEres.1
-  ## pooling model FEres vs. lag(FEres)
-  auxmod <- plm(FEres~FEres.1,data,model="pooling")
+  data <- data.frame(data[["(id)"]], data[["(time)"]],FEres = FEres, FEres.1 = FEres.1)
+  names(data)[c(1,2)] <- c("(id)","(time)")
+  auxmod <- plm(FEres~FEres.1, data = data, model = "pooling")
 
   ## calc. theoretical rho under H0: no serial corr. in errors
-#  t.<-attr(data, "pdim")$nT$T
-  t. <- pdim(data)$nT$T
+  t. <- pdim(x)$nT$T
   rho.H0 <- -1/(t.-1)
-
   ## test H0: rho=rho.H0 with HAC t-test (HC0-3 parm may be passed)
-  myvcov <- function(x) pvcovHC(x, method="arellano", ...)
+  myvcov <- function(x) vcovHC(x, method="arellano", ...)
+
   myH0 <- paste("FEres.1 = ", as.character(rho.H0), sep="")
-  linear.hypothesis(model=auxmod,"FEres.1=-0.125")
+  # args(linear.hypothesis) = test = c("F","Chisq") white.adjust = FALSE
   lhtest <- linear.hypothesis(model=auxmod, myH0, vcov.=myvcov, ...)
-#  print(lhtest)
-#  print(lhtest$Df)
-#  print(names(lhtest))
-#  print.default(lhtest)
+#  lhtest <- linear.hypothesis(model=auxmod, myH0, vcov.=myvcov(x), ...)
+
   ##(insert usual htest features)  
   FEARstat <- lhtest[2,3]
   names(FEARstat) <- dimnames(lhtest)[[2]][3]
@@ -229,8 +252,7 @@ pbsytest <- function (x, ...){
   UseMethod("pbsytest")
 }
 
-
-pbsytest.formula <- function(x,data,test=c("AR","RE","J"), index=NULL, ...) {
+pbsytest.formula <- function(x, data, ..., test=c("ar","re","j")) {
 
   ## Bera., Sosa-Escudero and Yoon type LM test for random effects
   ## under serial correlation (H0: no random effects) or the inverse;
@@ -241,62 +263,56 @@ pbsytest.formula <- function(x,data,test=c("AR","RE","J"), index=NULL, ...) {
 
   ######### from here generic testing interface from
   ######### plm to my code
-  formula <- x
 
-  data <- plm.data(data,index=index)
+  test <- match.arg(test)
+  
+  cl <- match.call(expand.dots = TRUE)
+  if (is.null(cl$model)) cl$model <- "pooling"
+  if (cl$model != "pooling") stop("pbsytest only relevant for pooling models")
+  names(cl)[2] <- "formula"
+  if (names(cl)[3] == "") names(cl)[3] <- "data"
+  m <- match(plm.arg,names(cl),0)
+  cl <- cl[c(1,m)]
+  cl[[1]] <- as.name("plm")
+  plm.model <- eval(cl,sys.frame(which=length(sys.calls())))
+  pbsytest(plm.model, test = test)
 
+}
+
+pbsytest.panelmodel <- function(x, test=c("ar","re","j"), ...){
+  poolres <- resid(x)
+  data <- model.frame(x)
   ## extract indices
-  index <- as.numeric(data[,1])
-  tindex <- as.numeric(data[,2])
-
-  ## reduce X,y to model matrix values (no NAs)
-  X <- model.matrix(formula,data=data)
-  y <- model.response(model.frame(formula,data=data))
-  ## reduce index accordingly
-  names(index) <- row.names(data)
-  ind <- index[which(names(index)%in%row.names(X))]
-  names(tindex) <- row.names(data)  
-  tind <- index[which(names(tindex)%in%row.names(X))]
-
-######### till here. 
-  ######### Add ordering here if needed.
-
+  index <- data[["(id)"]]
+  tindex <- data[["(time)"]]
+  
+  ## till here. 
+  ## ordering here if needed.
+  
   ## this needs ordering of obs. on time, regardless 
   ## whether before that on groups or after
-
-  ######### Ordering and numerosity check
-
+  
+  ## and numerosity check
+  
   ## order by group, then time
-  oo <- order(ind,tind)
-  X <- X[oo,]
-  y <- y[oo]
-  ind <- ind[oo]
-  tind <- tind[oo]
-
+  oo <- order(index,tindex)
+  ind <- index[oo]
+  tind <- tindex[oo]
+  poolres <- poolres[oo]
   ## det. number of groups and df
-  n <- length(unique(ind))
-  k <- dim(X)[[2]]
+  n <- length(unique(index))
+  k <- ncol(model.matrix(x))
   ## det. max. group numerosity
-  t <- max(tapply(X[,1],ind,length))
+  t <- max(pdim(x)$Tint$Ti)
   ## det. total number of obs. (robust vs. unbalanced panels)
   nT <- length(ind)
 
-  ######### End ordering block
-
-  ## begin test
-    
-            ## estimate pooled model
-  poolmod <- lm.fit(X,y)
-
-            ## extract pooled res. for BP statistic
-  poolres <- resid(poolmod)
-
             ## calc. A and B:
   S1 <- sum( tapply(poolres,ind,sum)^2 )
   S2 <- sum( poolres^2 )
             
   A <- S1/S2-1
-		
+
   unind <- unique(ind)
   uu <- rep(NA,length(unind))
   uu1 <- rep(NA,length(unind))
@@ -311,25 +327,22 @@ pbsytest.formula <- function(x,data,test=c("AR","RE","J"), index=NULL, ...) {
   B <- sum(uu1)/sum(uu)
   
   switch(match.arg(test),
-         AR={LM <- (n * t^2 * (B - (A/t))^2) / ((t-1)*(1-(2/t)))
+         ar ={LM <- (n * t^2 * (B - (A/t))^2) / ((t-1)*(1-(2/t)))
              df <- c(df=1)
-#                  names(LM) <- paste("Chisq(", df, ")", sep="")
              names(LM) <- "chisq"
              pLM <- pchisq(LM,df=1,lower.tail=FALSE)
              tname <- "Bera, Sosa-Escudero and Yoon locally robust test"
              myH0 <- "AR(1) errors sub random effects"
            },
-         RE={LM <- (A - 2*B) * sqrt( (n * t) / (2*(t-1)*(1-(2/t))) )
-#                  names(LM) <- "Z"
+         re={LM <- (A - 2*B) * sqrt( (n * t) / (2*(t-1)*(1-(2/t))) )
              names(LM) <- "z"
              df <- NULL
              pLM <- pnorm(LM,lower.tail=FALSE)
              tname <- "Bera, Sosa-Escudero and Yoon locally robust test"
              myH0 <- "random effects sub AR(1) errors"
            },              
-         J={LM <- (n * t^2) / (2*(t-1)*(t-2)) * (A^2 - 4*A*B + 2*t*B^2) 
+         j={LM <- (n * t^2) / (2*(t-1)*(t-2)) * (A^2 - 4*A*B + 2*t*B^2) 
             df <- c(df=2)
-#                 names(LM) <- paste("Chisq(", df, ")", sep="")
             names(LM) <- "chisq"
             pLM <- pchisq(LM,df=1,lower.tail=FALSE)
             tname <- "Baltagi and Li AR-RE joint test"
@@ -356,11 +369,19 @@ pdwtest <- function (x, ...)
     UseMethod("pdwtest")
 }
 
-pdwtest.formula <- function(x, data, model="random", index=NULL, ...) {
+pdwtest.formula <- function(x, data, ...) {
   ## formula method for pdwtest;
   ## defaults to a RE model
-  mymod <- plm(formula=x, data=data, model=model, index=index, ...)
-  pdwtest(mymod, ...)
+
+  cl <- match.call(expand.dots = TRUE)
+  if (is.null(cl$model)) cl$model <- "pooling"
+  names(cl)[2] <- "formula"
+  if (names(cl)[3] == "") names(cl)[3] <- "data"
+  m <- match(plm.arg,names(cl),0)
+  cl <- cl[c(1,m)]
+  cl[[1]] <- as.name("plm")
+  plm.model <- eval(cl,sys.frame(which=length(sys.calls())))
+  pdwtest(plm.model, ...)
 }
 
 pdwtest.panelmodel <- function(x,...) {
@@ -376,51 +397,61 @@ pdwtest.panelmodel <- function(x,...) {
   ## 3: apply bgtest() to auxiliary model and return the result
 
   ## retrieve demeaned data
-  demX <- x$model[[2]]
-  demy <- as.numeric(x$model[[1]])  ## pseries' give problems
+  demX <- model.matrix(x)
+  demy <- model.response(model.frame(x))
 
- ## dw test on the demeaned model:
-
- ## check package availability and load if necessary
+  ## dw test on the demeaned model:
+  
+  ## check package availability and load if necessary
   lm.ok <- require("lmtest")
   if(!lm.ok) stop("package lmtest is needed but not available")
+  ## ARtest is the bgtest, exception made for the method attribute
+  dots <- match.call(expand.dots=FALSE)[["..."]]
+  if (is.null(dots$order.by)) order.by <- NULL else order.by <- dots$order.by
+  if (is.null(dots$alternative)) alternative <- "greater" else alternative <- dots$alternative
+  if (is.null(dots$iterations)) iterations <- 15 else iterations <- dots$iterations
+  if (is.null(dots$exact)) exact <- NULL else exact <- dots$exact
+  if (is.null(dots$tol)) tol <- 1e-10 else tol <- dots$tol
+
+  # dwtest use diff, which should be the default and not the panelmodel method
+  attr(demy,"data") <- NULL
+  attr(demy, "class") <- attr(demy, "class")[-1]
   
- ## ARtest is the bgtest, exception made for the method attribute
-  ARtest <- dwtest(lm(demy~demX-1),...)
+  ARtest <- dwtest(lm(demy~demX-1), order.by = order.by, alternative = alternative,
+                   iterations = iterations, exact = exact, tol = tol)
+#  ARtest <- dwtest(lm(demy~demX-1))
+
   ARtest$method <- "Durbin-Watson test for serial correlation in panel models"
   ARtest$alternative <- "serial correlation in idiosyncratic errors"
   ARtest$data.name <- paste(deparse(x$call$formula))
-#  names(ARtest$statistic) <- "chisq"
   return(ARtest)
 }
 
 
-### pbltest
-
-
 ######### Baltagi and Li's LM_rho|mu ########
 ## ex Baltagi and Li (1995) Testing AR(1) against MA(1)...,
 ## JE 68, 133-151, test statistic (one-sided) is LM_4;
-## see also idem (1997), Monte carlo results..., 
+## see also idem (1997), Monte carlo results...,
 ## Annales d'Econometrie et Statistique 48, formula (8)
 
 ## from version 2: disposes of Kronecker products,
-## thus much faster and feasible on large NT (original 
-## is already infeasible for NT>3000, this takes 10'' 
+## thus much faster and feasible on large NT (original
+## is already infeasible for NT>3000, this takes 10''
 ## on N=3000, T=10 and even 20000x10 (55'') is no problem;
 ## lme() hits the memory limit at ca. 20000x20)
 
-pbltest <- function(x,data,alternative=c("twosided","onesided"), index=NULL, ...) {
- ## TODO: lme fails if there are any NAs in the data: reduce!
-
- ## usage: pbltest(modd.reg,data=dati0)
-
+pbltest <- function(x, data, alternative = c("twosided", "onesided"), index=NULL, ...) {
  ## this version (pbltest0) based on a "formula, pdataframe" interface
 
   data <- plm.data(data,index=index)
   gindex <- names(data)[1]
   tindex <- names(data)[2]
 
+  ## reduce X to model matrix value (no NAs)
+  X<-model.matrix(x,data=data)
+  ## reduce data accordingly
+  data <- data[which(row.names(data)%in%row.names(X)),]
+
   ## sort according to indices
   eval(parse(text=paste("data <- data[order(data$",gindex,",data$",tindex,"),]",sep="")))
 
@@ -444,7 +475,7 @@ pbltest <- function(x,data,alternative=c("twosided","onesided"), index=NULL, ...
     G[i-1,i] <- 1
     G[i,i-1] <- 1
     }
-  
+
   ## retrieve composite (=lowest level) residuals
   uhat <- residuals(mymod,level=0)
 
@@ -454,7 +485,7 @@ pbltest <- function(x,data,alternative=c("twosided","onesided"), index=NULL, ...
   uhat.i <- vector("list",n.)
   for(i in 1:n.) {
     uhat.i[[i]] <- uhat[t.*(i-1)+1:t.]
-    } 
+    }
   s2e <- rep(NA,n.)
   s21 <- rep(NA,n.)
   for(i in 1:n.) {
@@ -462,8 +493,8 @@ pbltest <- function(x,data,alternative=c("twosided","onesided"), index=NULL, ...
     s2e[i] <- as.numeric(crossprod(u.i,Et) %*% u.i)
     s21[i] <- as.numeric(crossprod(u.i,Jt) %*% u.i)
     }
-  sigma2.e <- sum(s2e) / (n.*(t.-1)) 
-  sigma2.1 <- sum(s21) / n. 
+  sigma2.e <- sum(s2e) / (n.*(t.-1))
+  sigma2.1 <- sum(s21) / n.
 
   ## calc. score under the null:
   star1 <- (Jt/sigma2.1 + Et/sigma2.e) %*% G %*% (Jt/sigma2.1 + Et/sigma2.e)
@@ -475,7 +506,7 @@ pbltest <- function(x,data,alternative=c("twosided","onesided"), index=NULL, ...
   star2 <- sum(star2)
   Drho <- (n.*(t.-1)/t.) * (sigma2.1-sigma2.e)/sigma2.1 + sigma2.e/2 * star2
   ## star2 is (crossprod(uhat, kronecker(In, star1)) %*% uhat)
-  
+
   ## components for the information matrix
   a <- (sigma2.e-sigma2.1)/(t.*sigma2.1)
   j.rr <- n. * (2 * a^2 * (t.-1)^2 + 2*a*(2*t.-3) + (t.-1))
@@ -484,13 +515,13 @@ pbltest <- function(x,data,alternative=c("twosided","onesided"), index=NULL, ...
   j.22 <- (n. * t.^2) / (2 * sigma2.1^2)
   j.23 <- (n. * t.) / (2 * sigma2.1^2)
   j.33 <- (n./2) * (1/sigma2.1^2 + (t.-1)/sigma2.e^2)
-  
+
   ## build up information matrix
   Jmat <- matrix(nrow=3,ncol=3)
   Jmat[1,] <- c(j.rr,j.12,j.13)
   Jmat[2,] <- c(j.12,j.22,j.23)
   Jmat[3,] <- c(j.13,j.23,j.33)
-  
+
   J11 <- n.^2 * t.^2 * (t.-1) / (det(Jmat) * 4*sigma2.1^2 * sigma2.e^2)
   ## this is the same as J11 <- solve(Jmat)[1,1], see BL page 73
 
@@ -507,11 +538,10 @@ pbltest <- function(x,data,alternative=c("twosided","onesided"), index=NULL, ...
          twosided = {
            LMr.m <- Drho^2 * J11
            pval <- pchisq(LMr.m,1,lower.tail=FALSE)
-#    names(LMr.m) <- "Chisq(1)"
            names(LMr.m) <- "chisq"
            parameter <- c(df=1)
            method1 <- "two-sided"
-           method2 <- "H0: rho = 0, HA: rho != 0" 
+           method2 <- "H0: rho = 0, HA: rho != 0"
          }
          )
   dname <- paste(deparse(substitute(x)))
@@ -529,55 +559,73 @@ pbltest <- function(x,data,alternative=c("twosided","onesided"), index=NULL, ...
   res
 }
 
-pwfdtest<-function(x, data, h0=c("fd","fe"), ...) {
+pwfdtest <- function(x, ...){
+  UseMethod("pwfdtest")
+}
+
+pwfdtest.formula <- function(x, data, ..., h0 = c("fd", "fe")){
+  cl <- match.call(expand.dots = TRUE)
+  if (is.null(cl$model)) cl$model <- "fd"
+  names(cl)[2] <- "formula"
+  if (names(cl)[3] == "") names(cl)[3] <- "data"
+  m <- match(plm.arg,names(cl),0)
+  cl <- cl[c(1,m)]
+  cl[[1]] <- as.name("plm")
+  plm.model <- eval(cl,sys.frame(which=length(sys.calls())))
+  pwfdtest(plm.model, ..., h0 = h0)
+}
+
+pwfdtest.panelmodel <- function(x, ..., h0=c("fd","fe")) {
   ## first-difference-based serial correlation test for panel models
   ## ref.: Wooldridge (2003), par. 10.6 
+
   if(!require(car)) stop("Library 'car' is needed")
 
   ## fetch fd residuals
-  fdmod<-plm(x,data,model="fd")
-  FDres<-fdmod$residuals
-
+  FDres <- resid(x)
   ## indices (full length! must reduce by 1st time period)
    ## this is an ad-hoc solution for the fact that the 'fd' model
    ## carries on the full indices while losing the first time period
-   time <- as.numeric(fdmod$indexes$time)
-   id <- as.numeric(fdmod$indexes$id)
-
+  time <- as.numeric(model.frame(x)[["(time)"]])
+  id <- as.numeric(model.frame(x)[["(id)"]])
 
    ## fetch dimensions and adapt to those of indices
-   n<-attr(fdmod,"pdim")$nT$n
+  pdim <- pdim(x)
+  n <- pdim$nT$n
+
  
    ## (re)create groupwise-separated index from 1 to nT 
    ## - dropping first time period
    ## - correcting Ti=Ti+1
-   Ti<-attr(fdmod,"pdim")$Tint$Ti
-   redind<-vector("list",n)
-   tfirst<-0
-   for(i in 1:n) {
-     redind[[i]]<-(tfirst+2):(tfirst+Ti[i]+1)
-     tfirst<-max(redind[[i]])
-    }
-
+  Ti <- pdim$Tint$Ti-1
+  
+  redind <- vector("list",n)
+  tfirst <- 0
+  for(i in 1:n) {
+    redind[[i]] <- (tfirst+2):(tfirst+Ti[i]+1)
+    tfirst <- max(redind[[i]])
+  }
    ## reduce indices by 1st time period
-   redind<-unlist(redind)
-   time<-time[redind]
-   id<-id[redind]
+  redind <- unlist(redind)
+  time <- time[redind]
+  id <- id[redind]
     
   N <- length(FDres)
+
   FDres.1 <- c(NA,FDres[1:(N-1)])
 
-  lagid <- id-c(NA,id[1:(N-1)])
+  lagid  <-  id-c(NA,id[1:(N-1)])
+
   FDres.1[lagid!=0] <- NA
 
+
   ## make (panel) dataframe for auxiliary regression
   auxdata <- as.data.frame(cbind(id,time))
-  auxdata$FDres<-FDres
-  auxdata$FDres.1<-FDres.1
-
+  auxdata$FDres <- FDres
+  auxdata$FDres.1 <- FDres.1
   ## pooling model FDres vs. lag(FDres),
   ## with intercept (might as well do it w.o.)
-  auxmod<-plm(FDres~FDres.1,auxdata,model="pooling")
+  auxmod <- plm(FDres ~ FDres.1, auxdata, model = "pooling")
 
   switch(match.arg(h0), 
              fd = {h0des<-"differenced"
@@ -590,15 +638,16 @@ pwfdtest<-function(x, data, h0=c("fd","fe"), ...) {
                    rho.H0 <- -0.5})
 
   ## test H0: rho=rho.H0 with HAC t-test (HC0-3 parm may be passed)
-  myvcov<-function(x) pvcovHC(x, method="arellano", ...)
+  myvcov <- function(x) vcovHC(x, method="arellano", ...)
 
-  myH0<-paste("FDres.1 = ", as.character(rho.H0), sep="")
-
-  lhtest<-linear.hypothesis(model=auxmod, myH0, vcov.=myvcov, ...)
+  myH0 <- paste("FDres.1 = ", as.character(rho.H0), sep="")
+  lhtest <- linear.hypothesis(model=auxmod, myH0, vcov.=myvcov, ...)
+cat("ca roule\n")
   
   ##(insert usual htest features)  
   FDARstat <- lhtest[2,3]
   names(FDARstat) <- dimnames(lhtest)[[2]][3] 
+  if (names(FDARstat)=="Chisq") names(FDARstat) <- "chisq"
   ## this is either 'F' or 'Chisq' and is the name of 3rd
   ## column because we are supplying a vcov matrix
   pFDAR<-lhtest[2,4]
